use bevy_ecs::prelude::Resource;
use std::{ops::Deref, sync::Arc};
use vulkano::{
    command_buffer::allocator::StandardCommandBufferAllocator,
    descriptor_set::allocator::StandardDescriptorSetAllocator,
    device::{Device, Queue},
    instance::Instance,
    memory::allocator::StandardMemoryAllocator,
};

#[derive(Resource)]
pub struct VulkanInstance(Arc<Instance>);

impl From<&Arc<Instance>> for VulkanInstance {
    fn from(instance: &Arc<Instance>) -> Self {
        Self(instance.clone())
    }
}

impl Deref for VulkanInstance {
    type Target = Arc<Instance>;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}

#[derive(Resource)]
pub struct VulkanDevice(Arc<Device>);

impl From<&Arc<Device>> for VulkanDevice {
    fn from(device: &Arc<Device>) -> Self {
        Self(device.clone())
    }
}

impl Deref for VulkanDevice {
    type Target = Arc<Device>;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}

#[derive(Resource)]
pub struct VulkanGraphicsQueue(Arc<Queue>);

impl From<&Arc<Queue>> for VulkanGraphicsQueue {
    fn from(queue: &Arc<Queue>) -> Self {
        Self(queue.clone())
    }
}

impl Deref for VulkanGraphicsQueue {
    type Target = Arc<Queue>;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}

#[derive(Resource)]
pub struct VulkanComputeQueue(Arc<Queue>);

impl From<&Arc<Queue>> for VulkanComputeQueue {
    fn from(queue: &Arc<Queue>) -> Self {
        Self(queue.clone())
    }
}

impl Deref for VulkanComputeQueue {
    type Target = Arc<Queue>;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}

#[derive(Resource)]
pub struct VulkanTransferQueue(Option<Arc<Queue>>);

impl From<Option<&Arc<Queue>>> for VulkanTransferQueue {
    fn from(queue: Option<&Arc<Queue>>) -> Self {
        Self(queue.cloned())
    }
}

impl Deref for VulkanTransferQueue {
    type Target = Option<Arc<Queue>>;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}

#[derive(Resource)]
pub struct VulkanMemoryAllocator(Arc<StandardMemoryAllocator>);

impl From<&Arc<StandardMemoryAllocator>> for VulkanMemoryAllocator {
    fn from(allocator: &Arc<StandardMemoryAllocator>) -> Self {
        Self(allocator.clone())
    }
}

impl Deref for VulkanMemoryAllocator {
    type Target = Arc<StandardMemoryAllocator>;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}

#[derive(Resource)]
pub struct VulkanCommandBufferAllocator(Arc<StandardCommandBufferAllocator>);

impl From<&Arc<StandardCommandBufferAllocator>> for VulkanCommandBufferAllocator {
    fn from(allocator: &Arc<StandardCommandBufferAllocator>) -> Self {
        Self(allocator.clone())
    }
}

impl Deref for VulkanCommandBufferAllocator {
    type Target = Arc<StandardCommandBufferAllocator>;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}

#[derive(Resource)]
pub struct VulkanDescriptorSetAllocator(Arc<StandardDescriptorSetAllocator>);

impl From<&Arc<StandardDescriptorSetAllocator>> for VulkanDescriptorSetAllocator {
    fn from(allocator: &Arc<StandardDescriptorSetAllocator>) -> Self {
        Self(allocator.clone())
    }
}

impl Deref for VulkanDescriptorSetAllocator {
    type Target = Arc<StandardDescriptorSetAllocator>;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}
